// SSLObject.cpp
//
// Creates and tests a C++ object that encapsulates
// the OpenSSL functionality.
//
// To compile you must download and have previously
// compiled the OpenSSL library. When using, be sure
// to place the OpenSSL libraries and/or executable files
// in your app's path!
// 
// For OpenSSL see:  www.openssl.org
//
// Kenn Scribner (based upon Simon Fell's OpenSSL Demo)
//
#include "SSLObject.h"

// Winsock version
const int CSSLObject::m_iWSVer = 0x00000202;

// SSL port identifier
const int CSSLObject::m_iSSLPort = 443;

// Read receive buffer (max) size (bytes)
const int CSSLObject::m_iRcvChunkSize = 8192;

CSSLObject::CSSLObject() :
	m_SSLContext(0),
	m_SSLConnection(0),
	m_SSLMethod(NULL),
	m_Certificate(NULL)
{
	// Standard constructor
}

CSSLObject::~CSSLObject()
{
}

int CSSLObject::Init()
{
	// Strt Winsock
	WSADATA wsaData;
	if ( WSAStartup(m_iWSVer,&wsaData) ) return WSAGetLastError();

	// Initialize SSL
	SSL_load_error_strings();
	SSLeay_add_ssl_algorithms();

	return 0;
}

int CSSLObject::Open(const std::string& strServer)
{
	// Locate the server
	struct sockaddr_in sa;
	struct hostent* ph = NULL;
	ph = gethostbyname(strServer.c_str());

	if ( !ph ) return -1;

	// Build the socket info
	memset(&sa,0,sizeof(sa)) ;
	sa.sin_family = ph->h_addrtype;
	sa.sin_port = htons(m_iSSLPort);
	memcpy(&(sa.sin_addr),ph->h_addr,ph->h_length);

	// Create the client socket
	m_Socket = socket(AF_INET,SOCK_STREAM,0); 
	if ( m_Socket == INVALID_SOCKET ) return WSAGetLastError();

	// Establish SSL context
	m_SSLMethod = SSLv2_client_method();
	m_SSLContext = SSL_CTX_new(m_SSLMethod);
	if ( !m_SSLContext ) return ERR_get_error();

	// Connect via SSL
	if ( connect(m_Socket,(struct sockaddr*)&sa,sizeof(sa)) == SOCKET_ERROR) return WSAGetLastError();

	// Start SSL negotiation
    m_SSLConnection = SSL_new(m_SSLContext);                         
	if ( !m_SSLConnection ) {
		closesocket(m_Socket);
		return ERR_get_error();
	} // if

	SSL_set_fd(m_SSLConnection,m_Socket);
	if ( SSL_connect(m_SSLConnection) == -1 ) return -1;

	return 0;
}

int CSSLObject::Read(const std::string& strRequest, std::string& strResults)
{
	// Send the packet
	int iResult = SSL_write(m_SSLConnection,strRequest.c_str(),strRequest.size());  
	if ( iResult == SOCKET_ERROR) {
		Close();
		return ERR_get_error();
	} // if

	// Clear return
	if ( !strResults.empty() ) strResults.erase();

	// Pull in the response chunks
	char szBuff[m_iRcvChunkSize];
	while ((iResult != SOCKET_ERROR) && (iResult != 0)) {
		iResult = SSL_read(m_SSLConnection,szBuff,sizeof(szBuff)-1);
		if ( iResult == SOCKET_ERROR ) {
			Close();
			return ERR_get_error();
		} // if
		else {
			szBuff[iResult] = '\0';
			strResults += szBuff;
		} // if
	} // while

	return iResult;
}

void CSSLObject::Close()
{
	// Shut down SSL
	SSL_shutdown(m_SSLConnection);
	SSL_free(m_SSLConnection);
	m_SSLConnection = 0;

	// Release the context
	SSL_CTX_free(m_SSLContext);
	m_SSLContext = 0;

	// Close the socket
	closesocket(m_Socket);

	// Clean up Winsock
	WSACleanup();
}


